@@ -16,7 +16,7 @@ module Patat.Presentation.Settings
 
     , ImageSettings (..)
 
-    , EvalSettingsMap
+    , EvalSettingsMap (..)
     , EvalSettingsContainer (..)
     , EvalSettings (..)
 
@@ -24,6 +24,8 @@ module Patat.Presentation.Settings
 
     , TransitionSettings (..)
 
+    , LinkSettings (..)
+
     , parseSlideSettings
     ) where
 
@@ -33,6 +35,9 @@ import           Control.Applicative    ((<|>))
 import           Control.Monad          (mplus, unless)
 import qualified Data.Aeson.Extended    as A
 import qualified Data.Aeson.TH.Extended as A
+import           Data.Bifunctor         (first)
+import           Data.CaseInsensitive   (CI)
+import qualified Data.CaseInsensitive   as CI
 import qualified Data.Foldable          as Foldable
 import           Data.Function          (on)
 import qualified Data.HashMap.Strict    as HMS
@@ -65,6 +70,7 @@ data PresentationSettings = PresentationSettings
     , psSyntaxDefinitions :: !(Maybe [FilePath])
     , psSpeakerNotes      :: !(Maybe SpeakerNotesSettings)
     , psTransition        :: !(Maybe TransitionSettings)
+    , psLinks             :: !(Maybe LinkSettings)
     } deriving (Eq, Show)
 
 
@@ -88,6 +94,7 @@ instance Semigroup PresentationSettings where
         , psSyntaxDefinitions = on (<>)  psSyntaxDefinitions l r
         , psSpeakerNotes      = on mplus psSpeakerNotes      l r
         , psTransition        = on mplus psTransition        l r
+        , psLinks             = on (<>)  psLinks             l r
         }
 
 
@@ -97,7 +104,7 @@ instance Monoid PresentationSettings where
     mempty  = PresentationSettings
                 Nothing Nothing Nothing Nothing Nothing Nothing Nothing Nothing
                 Nothing Nothing Nothing Nothing Nothing Nothing Nothing Nothing
-                Nothing
+                Nothing Nothing
 
 
 --------------------------------------------------------------------------------
@@ -204,6 +211,7 @@ defaultExtensionList = ExtensionList $
     , Pandoc.Ext_strikeout
     , Pandoc.Ext_superscript
     , Pandoc.Ext_subscript
+    , Pandoc.Ext_shortcut_reference_links
     ]
 
 
@@ -222,7 +230,15 @@ instance A.FromJSON ImageSettings where
 
 
 --------------------------------------------------------------------------------
-type EvalSettingsMap = HMS.HashMap T.Text EvalSettings
+newtype EvalSettingsMap = EvalSettingsMap (HMS.HashMap (CI T.Text) EvalSettings)
+    deriving (Eq, Show, Semigroup)
+
+
+--------------------------------------------------------------------------------
+instance A.FromJSON EvalSettingsMap where
+    parseJSON =
+        fmap (EvalSettingsMap . HMS.fromList . map (first CI.mk) . HMS.toList) .
+        A.parseJSON
 
 
 --------------------------------------------------------------------------------
@@ -252,6 +268,7 @@ data EvalSettings = EvalSettings
     , evalReveal    :: !Bool
     , evalContainer :: !EvalSettingsContainer
     , evalStderr    :: !Bool
+    , evalSyntax    :: !(Maybe (CI T.Text))
     } deriving (Eq, Show)
 
 
@@ -263,6 +280,7 @@ instance A.FromJSON EvalSettings where
         <*> deprecated "fragment" "reveal" True o
         <*> deprecated "wrap" "container" EvalContainerCode o
         <*> o A..:? "stderr" A..!= True
+        <*> (fmap CI.mk <$> o A..:? "syntax")
       where
         deprecated old new def obj = do
             mo <- obj A..:? old
@@ -295,6 +313,25 @@ instance A.FromJSON TransitionSettings where
         TransitionSettings <$> o A..: "type" <*> pure o
 
 
+--------------------------------------------------------------------------------
+data LinkSettings = LinkSettings
+    { lsOSC8 :: !(Maybe Bool)
+    } deriving (Eq, Show)
+
+
+--------------------------------------------------------------------------------
+instance Semigroup LinkSettings where
+    l <> r = LinkSettings
+        { lsOSC8 = on mplus lsOSC8 l r
+        }
+
+
+--------------------------------------------------------------------------------
+instance A.FromJSON LinkSettings where
+    parseJSON = A.withObject "FromJSON LinkSettings" $ \o ->
+        LinkSettings <$> o A..:? "osc8"
+
+
 --------------------------------------------------------------------------------
 $(A.deriveFromJSON A.dropPrefixOptions ''MarginSettings)
 $(A.deriveFromJSON A.dropPrefixOptions ''SpeakerNotesSettings)
